package test2;

/**/
import test2.analysis.*;
import test2.node.*;

import java.util.*;
import java.io.*;

/**
 * Caluculator class using DepthFirstAdapter and stack.
 *
 * @author  Katsumi Wasaki
 */
public class Calculator extends DepthFirstAdapter {

    //---------------------------------------------------------
    // operations for stack
    //---------------------------------------------------------

    LinkedList stack = new LinkedList();
    
    int getResult() {
        Integer val = (Integer) stack.getLast();
        return val.intValue();
    }
    
    void push(int val) {
        stack.add(new Integer(val));
    }
    
    int pop() {
        Integer val = (Integer) stack.removeLast();
        return val.intValue();
    }

    void printStack() {
	for (int pos=0;pos<stack.size();pos++){
	    Integer val = (Integer) stack.get(pos);
	    System.out.print(" " + val.intValue());
	}
	System.out.print("\n");
    }
    
    //---------------------------------------------------------
    // visitor functions (caluclation using stack)
    //---------------------------------------------------------

    //  expr = 
    //    {term}  term |
    //    {plus}  expr plus  term |
    //    {minus} expr minus term;

    public void caseAPlusExpr(APlusExpr node) {
	int left, right;

        node.getExpr().apply(this);
        node.getTerm().apply(this);
	right = pop();
	left = pop();
        push(left + right);
	System.out.print("+ ");
	printStack();
    }
    
    public void caseAMinusExpr(AMinusExpr node) {
	int left, right;

        node.getExpr().apply(this);
        node.getTerm().apply(this);
	right = pop();
	left = pop();
        push(left - right);
	System.out.print("- ");
	printStack();
    }
    
    //  term  = 
    //    {factor} factor |
    //    {mult}   term mult factor |
    //    {div}    term div  factor;

    public void caseAMultTerm(AMultTerm node) {
	int left, right;

        node.getTerm().apply(this);
        node.getFactor().apply(this);
	right = pop();
	left = pop();
        push(left * right);
	System.out.print("* ");
	printStack();
    }
    
    public void caseADivTerm(ADivTerm node) {
	int left, right;

        node.getTerm().apply(this);
        node.getFactor().apply(this);
	right = pop();
	left = pop();
        push(left / right);
	System.out.print("/ ");
	printStack();
    }
    
    //  factor = 
    //    {number} number | 
    //    {expr}   l_par expr r_par; 

    public void caseANumberFactor(ANumberFactor node) {
        int num = Integer.parseInt(node.getNumber().getText());
        push(num);
	System.out.print("N ");
	printStack();
    }

}
